#version 120
#extension GL_ARB_shader_texture_lod : enable
/*
!! DO NOT REMOVE !!
This code is from Chocapic13' shaders
Read the terms of modification and sharing before changing something below please !
!! DO NOT REMOVE !!
*/

	//#define RPSupport												//Enable resourcepack support for shaders, such as specular, bump mapping and POM.
	
	#define EmissiveRecolor											//Recolors some emissive blocks with torch color. Disabling this while using some torch color may give  weird results.
	#define LavaRecolor												//Recolors lava with original torch color.
	
	#define TorchColor_Original										//Yellow-orangeish torch color. Enable one only.
	//#define TorchColor_Bright										//White-orangeish torch color. Enable one only.
	//#define TorchColor_White										//White torch color. Enable one only.
	//#define TorchColor_End										//Purple torch color. Enable one only.
	//#define TorchColor_Water										//Blue torch color. Enable one only.
	//#define TorchColor_Leaf										//Green torch color. Enable one only.

	#define NORMAL_MAP_MAX_ANGLE 1.0
	#define POM_MAP_RES 128.0										//Set to resourcepack size. You don't need to change it if you don't use resourcepack with shader support. [16.0 32.0 64.0 128.0 256.0 512.0]
	#define POM_DEPTH 0.10											//Set POM's depth. [0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50]

/* Here, intervalMult might need to be tweaked per texture pack.
   The first two numbers determine how many samples are taken per fragment.  They should always be the equal to eachother.
   The third number divided by one of the first two numbers is inversely proportional to the range of the height-map. */

const vec3 intervalMult = vec3(1.0, 1.0, 1.0/POM_DEPTH)/POM_MAP_RES * 1.0;

const float MAX_OCCLUSION_DISTANCE = 22.0;
const float MIX_OCCLUSION_DISTANCE = 18.0;
const int   MAX_OCCLUSION_POINTS   = int(POM_MAP_RES);


const int RGBA16 = 3;
const int RGB16 = 2;
const int RGBA8 = 1;
const int R8 = 0;

const int gdepthFormat = RGBA16;
const int gnormalFormat = RGB16;
const int compositeFormat = RGBA16;
const int gaux2Format = RGBA16;
const int gcolorFormat = RGBA8;

const int GL_EXP = 2048;
const int GL_LINEAR = 9729;
const float bump_distance = 64.0;		//bump render distance: tiny = 32, short = 64, normal = 128, far = 256
const float pom_distance = 32.0;		//POM render distance: tiny = 32, short = 64, normal = 128, far = 256
const float fademult = 0.1;

varying vec2 lmcoord;
varying vec4 color;
varying float mat;
varying float brmult;
varying vec2 texcoord;

varying vec3 normal;

uniform sampler2D texture;
uniform sampler2D noisetex;
uniform sampler2D normals;

#ifdef RPSupport
varying vec4 vtexcoordam; // .st for add, .pq for mul
varying vec4 vtexcoord;
varying vec3 wpos;
varying vec3 tangent;
varying vec3 binormal;
varying vec3 viewVector;
varying float dist;
uniform sampler2D specular;
#endif

uniform int worldTime;
uniform float wetness;
uniform float frameTimeCounter;

const float mincoord = 1.0/4096.0;
const float maxcoord = 1.0-mincoord;

#ifdef RPSupport
vec2 dcdx = dFdx(vtexcoord.st*vtexcoordam.pq);
vec2 dcdy = dFdy(vtexcoord.st*vtexcoordam.pq);

vec4 readTexture(in vec2 coord)
{
	return texture2DGradARB(texture,fract(coord)*vtexcoordam.pq+vtexcoordam.st,dcdx,dcdy);
}

vec4 readNormal(in vec2 coord)
{
	return texture2DGradARB(normals,fract(coord)*vtexcoordam.pq+vtexcoordam.st,dcdx,dcdy);
}
#endif

//Texture Recolor Colors
#ifdef TorchColor_Original
vec3 TorchColor = vec3(0.7,0.28,0.04);
#endif
#ifdef TorchColor_Bright
vec3 TorchColor = vec3(0.7,0.35,0.15);
#endif
#ifdef TorchColor_White
vec3 TorchColor = vec3(0.4);
#endif
#ifdef TorchColor_End
vec3 TorchColor = vec3(0.65,0.27,0.65);
#endif
#ifdef TorchColor_Water
vec3 TorchColor = vec3(0.1,0.35,1.0);
#endif
#ifdef TorchColor_Leaf
vec3 TorchColor = vec3(0.15,0.75,0.15);
#endif

//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////
//////////////////////////////VOID MAIN//////////////////////////////

void main() {
	vec4 colorAlbedo = texture2D(texture, texcoord);

#ifdef RPSupport
vec2 adjustedTexCoord = vtexcoord.st*vtexcoordam.pq+vtexcoordam.st;
if (dist < MAX_OCCLUSION_DISTANCE) {
		if ( viewVector.z < 0.0 && readNormal(vtexcoord.st).a < 0.99 && readNormal(vtexcoord.st).a > 0.01)
	{
		vec3 interval = viewVector.xyz * intervalMult;
		vec3 coord = vec3(vtexcoord.st, 1.0);
		for (int loopCount = 0;
				(loopCount < MAX_OCCLUSION_POINTS) && (readNormal(coord.st).a < coord.p);
				++loopCount) {
			coord = coord+interval;

		}
		if (coord.t < mincoord) {
			if (readTexture(vec2(coord.s,mincoord)).a == 0.0) {
				coord.t = mincoord;
				discard;
			}
		}
		adjustedTexCoord = mix(fract(coord.st)*vtexcoordam.pq+vtexcoordam.st , adjustedTexCoord , max(dist-MIX_OCCLUSION_DISTANCE,0.0)/(MAX_OCCLUSION_DISTANCE-MIX_OCCLUSION_DISTANCE));
	}

}
	colorAlbedo = texture2DGradARB(texture, adjustedTexCoord, dcdx, dcdy);
#endif
	
	vec4 frag2 = vec4(normal*0.5 + 0.5, 1.0f);
	vec4 spec = vec4(0.0);
	
	#ifdef RPSupport
	vec3 bump = texture2DGradARB(normals, adjustedTexCoord, dcdx, dcdy).rgb*2.0-1.0;
	float bumpmult = NORMAL_MAP_MAX_ANGLE*(1.0-wetness*lmcoord.t*0.9);
	bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
	mat3 tbnMatrix = mat3(  tangent.x, binormal.x, normal.x,
							tangent.y, binormal.y, normal.y,
							tangent.z, binormal.z, normal.z);

	frag2 = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
	
	spec = texture2DGradARB(specular, adjustedTexCoord.st, dcdx, dcdy);
	#endif
	
	float emissivecolored = float(mat > 0.61 && mat < 0.64);
	float islava = float(mat > 0.67 && mat < 0.69);
	
	#ifdef EmissiveRecolor
	if (emissivecolored > 0.9){
		float ec = clamp(pow(length(colorAlbedo.rgb),1.4),0,2.2);
		colorAlbedo.rgb = clamp(ec*TorchColor*0.4+pow(ec,1.0)*0.35,vec3(0),vec3(2.2));
		}
	#endif
	#ifdef LavaRecolor
	if (islava > 0.9){
		float ec = clamp(pow(length(colorAlbedo.rgb),1.4),0,2.2);
		colorAlbedo.rgb = clamp(ec*vec3(0.7,0.4,0.1)+ec*0.1,vec3(0),vec3(2.2));
		}
	#endif
	
	colorAlbedo.rgb = pow(pow(colorAlbedo.rgb,vec3(2.2))*(1.0+brmult),vec3(1.0/2.2));
	
/* DRAWBUFFERS:0246 */

	gl_FragData[0] = colorAlbedo*color;
	gl_FragData[1] = frag2;
	gl_FragData[2] = vec4(lmcoord.t, mat, lmcoord.s, 1.0);
	gl_FragData[3] = spec;
}
